"use strict";

var _chai = require("chai");
var _marcRecord = require("@natlibfi/marc-record");
var _doubleCommas = _interopRequireDefault(require("../src/double-commas"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
describe('double-commas', () => {
  it('Creates a validator', async () => {
    const validator = await (0, _doubleCommas.default)();
    (0, _chai.expect)(validator).to.be.an('object').that.has.any.keys('description', 'validate');
    (0, _chai.expect)(validator.description).to.be.a('string');
    (0, _chai.expect)(validator.validate).to.be.a('function');
  });
  describe('#validate', () => {
    it('Finds the record valid', async () => {
      const validator = await (0, _doubleCommas.default)();
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '700',
          subfields: [{
            code: 'e',
            value: 'foo,bar'
          }]
        }]
      });
      const result = await validator.validate(record);
      (0, _chai.expect)(result).to.eql({
        valid: true
      });
    });
    it('Finds the record invalid', async () => {
      const validator = await (0, _doubleCommas.default)();
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '700',
          subfields: [{
            code: 'e',
            value: 'foo,,bar'
          }]
        }]
      });
      const result = await validator.validate(record);
      (0, _chai.expect)(result).to.eql({
        valid: false
      });
    });
  });
  describe('#fix', () => {
    it('Fixes the record', async () => {
      const validator = await (0, _doubleCommas.default)();
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '700',
          subfields: [{
            code: 'e',
            value: 'foo,,bar'
          }]
        }]
      });
      await validator.fix(record);
      (0, _chai.expect)(record.fields).to.eql([{
        tag: '700',
        ind1: ' ',
        ind2: ' ',
        subfields: [{
          code: 'e',
          value: 'foo,bar'
        }]
      }]);
    });
  });
});
//# sourceMappingURL=double-commas.spec.js.map