"use strict";

var _chai = require("chai");
var _marcRecord = require("@natlibfi/marc-record");
var _duplicatesInd = _interopRequireDefault(require("./duplicates-ind1"));
var _fixura = require("@natlibfi/fixura");
var _fixugen = _interopRequireDefault(require("@natlibfi/fixugen"));
var _debug = _interopRequireDefault(require("debug"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
(0, _fixugen.default)({
  callback,
  path: [__dirname, '..', 'test-fixtures', 'duplicates-ind1'],
  useMetadataFile: true,
  recurse: false,
  fixura: {
    reader: _fixura.READERS.JSON
  },
  mocha: {
    before: () => testValidatorFactory()
  }
});
const debug = (0, _debug.default)('@natlibfi/marc-record-validators-melinda/duplicates-ind1:test');
async function testValidatorFactory() {
  const validator = await (0, _duplicatesInd.default)(/^500$/u);
  (0, _chai.expect)(validator).to.be.an('object').that.has.any.keys('description', 'validate');
  (0, _chai.expect)(validator.description).to.be.a('string');
  (0, _chai.expect)(validator.validate).to.be.a('function');
}
async function callback({
  getFixture,
  tagPattern,
  fix = false
}) {
  const validator = await (0, _duplicatesInd.default)(new RegExp(tagPattern, 'u'));
  const record = new _marcRecord.MarcRecord(getFixture('record.json'));
  const expectedResult = getFixture('expectedResult.json');
  if (!fix) {
    const result = await validator.validate(record);
    return (0, _chai.expect)(result).to.eql(expectedResult);
  }
  const fixedRecord = await validator.fix(record);
  debug(fixedRecord);
  (0, _chai.expect)(record.fields).to.eql(expectedResult);
}
//# sourceMappingURL=duplicates-ind1.spec.js.map