"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
var _clone = _interopRequireDefault(require("clone"));
var _utils = require("./utils");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
//import createDebugLogger from 'debug';

// Author(s): Nicholas Volk
function _default() {
  return {
    description: 'Normalize " ; " separators as " -- "',
    validate,
    fix
  };
  function fix(record) {
    record.fields.forEach(field => {
      fix505(field);
    });
    const res = {
      message: [],
      fix: [],
      valid: true
    };
    return res;
  }
  function validate(record) {
    const res = {
      message: []
    };
    record.fields?.forEach(field => {
      validateField(field, res);
    });
    res.valid = !(res.message.length >= 1); // eslint-disable-line functional/immutable-data
    return res;
  }
  function validateField(field, res) {
    const orig = (0, _utils.fieldToString)(field);
    const normalizedField = fix505((0, _clone.default)(field));
    const mod = (0, _utils.fieldToString)(normalizedField);
    if (orig !== mod) {
      // Fail as the input is "broken"/"crap"/sumthing
      res.message.push(`'TODO: ${orig}' => '${mod}'`); // eslint-disable-line functional/immutable-data
      return;
    }
    return;
  }
}
function field505FixSubfieldA(field) {
  const a = field.subfields.filter(sf => sf.code === 'a');
  a.forEach(sf => fixSubfieldA(sf));
  function fixSubfieldA(a) {
    a.value = a.value.replace(/ ; /ug, ' -- '); // eslint-disable-line functional/immutable-data
  }
}
function field505FixSubfieldTRG(field) {
  // Modify subfield if next subfield is $t:
  const subfieldsThatWillBeModified = field.subfields.filter((sf, i) => i + 1 < field.subfields.length && field.subfields[i + 1].code === 't');
  subfieldsThatWillBeModified.forEach(sf => fixSubfieldThatPrecedesT(sf));
  function fixSubfieldThatPrecedesT(sf) {
    if (!sf.value) {
      return;
    }
    sf.value = sf.value.replace(/ ;$/u, ' --'); // eslint-disable-line functional/immutable-data
  }
}
function fix505(field) {
  if (field.tag !== '505' || !field.subfields) {
    return field;
  }
  field505FixSubfieldTRG(field);
  field505FixSubfieldA(field);
  return field;
}
//# sourceMappingURL=field-505-separators.js.map