"use strict";

var _chai = require("chai");
var _marcRecord = require("@natlibfi/marc-record");
var _fixLanguageCodes = _interopRequireDefault(require("./fix-language-codes"));
var _fixura = require("@natlibfi/fixura");
var _fixugen = _interopRequireDefault(require("@natlibfi/fixugen"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
(0, _fixugen.default)({
  callback,
  path: [__dirname, '..', 'test-fixtures', 'fix-language-codes'],
  useMetadataFile: true,
  recurse: false,
  fixura: {
    reader: _fixura.READERS.JSON
  },
  mocha: {
    before: () => testValidatorFactory()
  }
});
async function testValidatorFactory() {
  const validator = await (0, _fixLanguageCodes.default)();
  (0, _chai.expect)(validator).to.be.an('object').that.has.any.keys('description', 'validate');
  (0, _chai.expect)(validator.description).to.be.a('string');
  (0, _chai.expect)(validator.validate).to.be.a('function');
}
async function callback({
  getFixture,
  fix = false
}) {
  const validator = await (0, _fixLanguageCodes.default)();
  const record = new _marcRecord.MarcRecord(getFixture('record.json'));
  const expectedResult = getFixture('expectedResult.json');
  // console.log(expectedResult); // eslint-disable-line

  if (!fix) {
    const result = await validator.validate(record);
    (0, _chai.expect)(result).to.eql(expectedResult);
    return;
  }
  await validator.fix(record);
  (0, _chai.expect)(record).to.eql(expectedResult);
}
//# sourceMappingURL=fix-language-codes.spec.js.map