"use strict";

var _chai = _interopRequireDefault(require("chai"));
var _chaiAsPromised = _interopRequireDefault(require("chai-as-promised"));
var _marcRecord = require("@natlibfi/marc-record");
var _fixedFields = _interopRequireDefault(require("../src/fixed-fields"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const {
  expect
} = _chai.default;
_chai.default.use(_chaiAsPromised.default);
describe('fixed-fields: language', () => {
  it('Creates a validator', async () => {
    const validator = await (0, _fixedFields.default)([]);
    expect(validator).to.be.an('object').that.has.any.keys('description', 'validate');
    expect(validator.description).to.be.a('string');
    expect(validator.validate).to.be.a('function');
  });
  it('Throws an error when configuration is not provided', () => {
    try {
      (0, _fixedFields.default)();
    } catch (error) {
      expect(error).to.be.an('error').with.property('message', 'No configuration provided');
    }
  });
  describe('#validate', () => {
    it('Finds the record valid', async () => {
      const validator = await (0, _fixedFields.default)([{
        leader: true,
        length: 6,
        rules: [{
          position: [0, 6],
          pattern: /[abcdefg]/u
        }]
      }, {
        tag: /^FOO$/u,
        length: 12,
        rules: [{
          position: 0,
          pattern: /f/u
        }]
      }, {
        tag: /^BAR$/u,
        length: 6,
        rules: [{
          position: 0,
          pattern: /[fb]/u
        }, {
          position: 1,
          pattern: /a/u,
          dependencies: [{
            position: 0,
            pattern: /b/u
          }]
        }, {
          position: [2, 3],
          pattern: /u/u,
          dependencies: [{
            position: 0,
            pattern: /[^b]/u
          }]
        }]
      }]);
      const record = new _marcRecord.MarcRecord({
        leader: 'bacgfe',
        fields: [{
          tag: 'FOO',
          value: 'foobarfoobar'
        }, {
          tag: 'BAR',
          value: 'bauuoo'
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.be.an('object').and.to.include({
        valid: true
      });
    });
    it('Finds the record invalid', async () => {
      const validator = await (0, _fixedFields.default)([{
        leader: true,
        length: 6,
        rules: [{
          position: [0, 6],
          pattern: /[abcdefg]/u
        }]
      }, {
        tag: /^FOO$/u,
        length: 12,
        rules: [{
          position: 0,
          pattern: /f/u
        }]
      }, {
        tag: /^BAR$/u,
        length: 6,
        rules: [{
          position: 0,
          pattern: /[fb]/u
        }, {
          position: 1,
          pattern: /a/u,
          dependencies: [{
            position: 0,
            pattern: /b/u
          }]
        }, {
          position: [2, 3],
          pattern: /u/u,
          dependencies: [{
            position: 0,
            pattern: /[^a]/u
          }]
        }]
      }, {
        tag: /^FUB$/u,
        length: 5
      }]);
      const record = new _marcRecord.MarcRecord({
        leader: 'bacxfe',
        fields: [{
          tag: 'FOO',
          value: 'barfoofoobar'
        }, {
          tag: 'BAR',
          value: 'burfoo'
        }, {
          tag: 'FUB',
          value: 'foo'
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: false,
        messages: ['Leader has invalid values at positions: 3 (Rule index 0)', 'Field FOO has invalid values at positions: 0 (Rule index 0)', 'Field BAR has invalid values at positions: 1 (Rule index 1)', 'Field BAR has invalid values at positions: 2,3 (Rule index 2)', 'Field FUB has invalid length']
      });
    });
  });
});
//# sourceMappingURL=fixed-fields.spec.js.map