"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
var _marcRecord = require("@natlibfi/marc-record");
/**
 * This validator de-duplicates identical fields in a record.
 */

function _default() {
  return {
    description: 'Handles identical duplicate in record fields',
    validate,
    fix
  };
  function validate(record) {
    const uniq = uniqWith(record.fields);
    const valid = uniq.length === record.fields.length;
    const messages = record.fields.filter(tag => !uniq.includes(tag)).map(obj => `Field ${obj.tag} has duplicates`);
    return valid ? {
      valid,
      messages: []
    } : {
      valid,
      messages
    };
  }
  function fix(record) {
    record.fields.filter(tag => !uniqWith(record.fields).includes(tag)).forEach(tag => record.removeField(tag));
  }
  function uniqWith(fields) {
    return fields.reduce((uniq, field) => {
      if (!uniq.some(f => _marcRecord.MarcRecord.isEqual(f, field))) {
        // eslint-disable-line functional/no-conditional-statements
        uniq.push(field); // eslint-disable-line functional/immutable-data
      }
      return uniq;
    }, []);
  }
}
//# sourceMappingURL=identical-fields.js.map