"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeOrAddSubfield = mergeOrAddSubfield;
var _debug = _interopRequireDefault(require("debug"));
var _normalizeFieldForComparison = require("../normalizeFieldForComparison.js");
var _normalizeIdentifiers = require("../normalize-identifiers");
var _utils = require("../utils.js");
var _mergeSubfield = require("./mergeSubfield.js");
var _sortSubfields = require("../sortSubfields");
var _worldKnowledge = require("./worldKnowledge.js");
var _subfield6Utils = require("../subfield6Utils.js");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
//'./sortSubfields.js';

const debug = (0, _debug.default)('@natlibfi/melinda-marc-record-merge-reducers:mergeOrAddSubfield');
//const debugData = debug.extend('data');
const debugDev = debug.extend('dev');
function catalogingSourceModifyingAgencyCandIsOriginalCatalogingSourceAgencyInTargetField(targetField, candSubfieldData) {
  if (targetField.tag !== '040' || candSubfieldData.code !== 'd') {
    return false;
  }
  (0, _utils.nvdebug)(`${(0, _utils.fieldToString)(targetField)} vs $d ${candSubfieldData.originalValue}}`, debugDev);
  // Add hard-coded exceptions here
  if (targetField.subfields.some(sf => sf.code === 'a' && sf.value === candSubfieldData.originalValue)) {
    (0, _utils.nvdebug)('040‡d matched 040‡a', debugDev);
    return true;
  }
  return false;
}
function ennakkotietoInSubfieldG(candSubfieldData) {
  if ((0, _normalizeFieldForComparison.isEnnakkotietoSubfieldG)({
    'code': candSubfieldData.code,
    'value': candSubfieldData.originalValue
  })) {
    // Skip just ‡g subfield or the whole field?
    // We decided to skip just this subfield. We want at least $0 and maybe even more from ennakkotieto.
    debugDev(`Skip '‡g ${candSubfieldData.originalValue}'`);
    return true;
  }
  return false;
}
function mergeOrAddSubfieldNotRequiredSpecialCases(targetField, candSubfieldData) {
  // Don't bring WHATEVER<KEEP> from source 7XX to base 1XX.
  // Exceptionally we can merge <KEEP>ed 7XX with un-<KEEP>ed 1XX as 1XX should not use <KEEP>s.
  if (targetField.tag.charAt(0) === '1' && candSubfieldData.tag.charAt(0) === '7' && candSubfieldData.code === '9' && candSubfieldData.originalValue.match(/<KEEP>/u)) {
    return true;
  }

  // Don't add 264$b 'Kustannuspaikka tuntematon' etc
  if (!(0, _worldKnowledge.valueCarriesMeaning)(targetField.tag, candSubfieldData.code, candSubfieldData.normalizedValue)) {
    return true;
  }

  // Don't add $0 subfields that mean the same even if they look different:
  const alephIdentifierType = (0, _normalizeIdentifiers.normalizeAs)(targetField.tag, candSubfieldData.code);
  if (alephIdentifierType !== undefined) {
    const normalizedSubfieldValue = (0, _normalizeIdentifiers.normalizeControlSubfieldValue)(candSubfieldData.originalValue, alephIdentifierType);
    if (targetField.subfields.some(sf => (0, _normalizeIdentifiers.normalizeControlSubfieldValue)(sf.value) === normalizedSubfieldValue && sf.code === candSubfieldData.code)) {
      return true;
    }
  }
  return false;
}
function skipNormalizedComparison(tag, subfieldCode, subfieldValue) {
  if (tag === '020' && subfieldCode === 'a') {
    return true;
  }
  // Hackish: we want 'ntamo' to win 'Ntamo'...
  // If there are other similar excepting put them into an array.
  if (['110', '610', '710', '810'].includes(tag) && subfieldCode === 'a' && subfieldValue.substring(0, 5) === 'ntamo') {
    return true;
  }
  return false;
}
function mergeOrAddSubfieldNotRequired(targetField, candSubfieldData) {
  if (catalogingSourceModifyingAgencyCandIsOriginalCatalogingSourceAgencyInTargetField(targetField, candSubfieldData) || ennakkotietoInSubfieldG(candSubfieldData)) {
    return true;
  }
  if (mergeOrAddSubfieldNotRequiredSpecialCases(targetField, candSubfieldData)) {
    return true;
  }
  const relevantTargetSubfields = targetField.subfields.filter(sf => sf.code === candSubfieldData.code);
  // Target field does not have this subfield yet:
  if (relevantTargetSubfields.length === 0) {
    return false;
  }
  (0, _utils.nvdebug)(`     Look for identical subfields in '${(0, _utils.fieldToString)(targetField)}' using`, debugDev);
  (0, _utils.nvdebug)(`      ORIG.    ‡${candSubfieldData.code} ${candSubfieldData.originalValue}`, debugDev);
  (0, _utils.nvdebug)(`      NO-PUNC  ‡${candSubfieldData.code} ${candSubfieldData.punctuationlessValue}`, debugDev);
  if (relevantTargetSubfields.some(sf => sf.code === candSubfieldData.code && sf.value === candSubfieldData.originalValue)) {
    return true;
  }
  if (relevantTargetSubfields.some(sf => sf.code === candSubfieldData.code && sf.value === candSubfieldData.punctuationlessValue)) {
    return true;
  }
  if (!skipNormalizedComparison(targetField.tag, candSubfieldData.code, candSubfieldData.originalValue)) {
    const normalizedTargetField = (0, _normalizeFieldForComparison.cloneAndNormalizeFieldForComparison)(targetField);
    (0, _utils.nvdebug)(`     Look for identical normalized subfields in '${(0, _utils.fieldToString)(normalizedTargetField)}'`, debugDev);
    (0, _utils.nvdebug)(`      NO-PUNC ‡${candSubfieldData.code} ${candSubfieldData.normalizedValue})`, debugDev);
    if (normalizedTargetField.subfields.some(sf => sf.code === candSubfieldData.code && sf.value === candSubfieldData.normalizedValue)) {
      // Subfield with identical normalized value exists. Do nothing.
      // Not ideal 382‡n subfields, I guess... Nor 505‡trg repetitions... These need to be fixed...
      return true;
    }
  }
  return false; // (note that this is a double negation: not required is false)
}
function addSubfield(targetField, candSubfield) {
  (0, _utils.nvdebug)(` Added subfield '${(0, _utils.subfieldToString)(candSubfield)}' to field`, debugDev);
  // Add subfield to the end of all subfields. NB! Implement a separate function that does this + subfield reordering somehow...
  targetField.subfields.push(candSubfield); // eslint-disable-line functional/immutable-data

  targetField.merged = 1; // eslint-disable-line functional/immutable-data

  setPunctuationFlag(targetField, candSubfield);
  (0, _sortSubfields.sortAdjacentSubfields)(targetField);
}
function setPunctuationFlag(field, addedSubfield) {
  if ((0, _utils.isControlSubfieldCode)(addedSubfield.code)) {
    // These are never punctuation related
    return;
  }
  field.useExternalEndPunctuation = 1; // eslint-disable-line functional/immutable-data
}
function resetPaired880(candFieldPair880, targetField, punctlessCandSubfield) {
  // No relevant:
  if (punctlessCandSubfield.code !== '6') {
    return;
  }
  if (targetField.tag === '880') {
    return;
  }
  // NB! $6 comes first:
  if (candFieldPair880 === undefined || !candFieldPair880.subfields || candFieldPair880.subfields[0].code !== '6') {
    return;
  }
  (0, _utils.nvdebug)(`880 contents: ${(0, _utils.fieldToString)(candFieldPair880)}`, debugDev);
  (0, _subfield6Utils.resetSubfield6Tag)(candFieldPair880.subfields[0], targetField.tag);
}
function mergeOrAddSubfield(targetField, candSubfieldData, candFieldPairs880 = []) {
  const candSubfieldAsString = `${candSubfieldData.code} ${candSubfieldData.originalValue}`;
  (0, _utils.nvdebug)(`   Q: mergeOrAddSubfield '${candSubfieldAsString}'\n      with field '${(0, _utils.fieldToString)(targetField)}'?`, debugDev);
  if (mergeOrAddSubfieldNotRequired(targetField, candSubfieldData)) {
    (0, _utils.nvdebug)(`    A: No. No need to merge nor to add the subfield '${candSubfieldAsString}'`, debugDev);
    return;
  }
  const candSubfield = {
    'code': candSubfieldData.code,
    'value': candSubfieldData.punctuationlessValue
  };

  // Currently only for X00$d 1984- => 1984-2000 type of changes, where source version is better that what base has.
  // It all other cases the original subfield is kept.
  const original = (0, _utils.fieldToString)(targetField);
  if ((0, _mergeSubfield.mergeSubfield)(targetField, candSubfield)) {
    // We might need the normalizedCandSubfield later on
    mergeSubfieldPostprocessor();
    return;
  }

  // Subfield codes missing from the original record can be added by default:
  if (addSubfieldWithPreviouslyUnseenSubfieldCode()) {
    return;
  }

  // melindaCustomMergeFields.json tells us whether the subfield is repeatable or not:
  if ((0, _utils.subfieldIsRepeatable)(targetField.tag, candSubfield.code)) {
    // We don't want to add multiple, say, 260$c
    if (['260', '264'].includes(targetField.tag)) {
      (0, _utils.nvdebug)(`    A: Exceptionally skip repeatable existing subfield '${(0, _utils.subfieldToString)(candSubfield)}'`, debugDev);
      return;
    }
    (0, _utils.nvdebug)(`    A: Yes. Add repeatable subfield '${(0, _utils.subfieldToString)(candSubfield)}'`, debugDev);
    targetField.merged = 1; // eslint-disable-line functional/immutable-data
    setPunctuationFlag(targetField, candSubfield);
    addSubfield(targetField, candSubfield);
    return;
  }
  (0, _utils.nvdebug)(`    A: No. Non-repeatable subfield '${(0, _utils.subfieldToString)(candSubfield)}'`, debugDev);
  return;
  function mergeSubfieldPostprocessor() {
    if (original !== (0, _utils.fieldToString)(targetField)) {
      (0, _utils.nvdebug)(`    A: Merge. Subfield '${candSubfieldAsString}' replaces the original subfield.`, debugDev);
      targetField.merged = 1; // eslint-disable-line functional/immutable-data
      setPunctuationFlag(targetField, candSubfield);
      return;
    }
    (0, _utils.nvdebug)(`      A: No. Field ${original} already had the same or a synonymous or a better merge candidate than our subfield '${candSubfieldAsString}'.`, debugDev);
    return;
  }
  function addSubfieldWithPreviouslyUnseenSubfieldCode() {
    if (!(0, _utils.fieldHasSubfield)(targetField, candSubfield.code)) {
      (0, _utils.nvdebug)(`    A: Yes. Add previously unseen subfield '${(0, _utils.subfieldToString)(candSubfield)}'`, debugDev);
      targetField.merged = 1; // eslint-disable-line functional/immutable-data
      setPunctuationFlag(targetField, candSubfield);
      candFieldPairs880.forEach(pair => resetPaired880(pair, targetField, candSubfield));
      addSubfield(targetField, candSubfield);
      return true;
    }
    return false;
  }
}
//# sourceMappingURL=mergeOrAddSubfield.js.map