"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fieldRemoveDuplicateSubfields = fieldRemoveDuplicateSubfields;
exports.recordRemoveDuplicateSubfieldsFromFields = recordRemoveDuplicateSubfieldsFromFields;
var _utils = require("../utils.js");
var _debug = _interopRequireDefault(require("debug"));
var _normalizeFieldForComparison = require("../normalizeFieldForComparison.js");
var _sortSubfields = require("../sortSubfields");
var _punctuation = require("../punctuation2");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// NB This should be moved and converted to a validator/fixer as well...
const debug = (0, _debug.default)('@natlibfi/marc-record-validators-melinda:merge-fields:removeDuplicateSubfields');
//const debugData = debug.extend('data');
const debugDev = debug.extend('dev');
function recordRemoveDuplicateSubfieldsFromFields(record) {
  record.fields.forEach(field => fieldRemoveDuplicateSubfields(field));
}
function fieldRemoveDuplicateSubfields(field) {
  // Skip bad (382, 505) and risky (264 ...) stuff: 382$n, 505$r, others...
  if (!field.subfields || ['264', '300', '382', '505'].includes(field.tag)) {
    return;
  }
  const strippedField = (0, _normalizeFieldForComparison.cloneAndRemovePunctuation)(field); // make punctuation-less version
  /* eslint-disable */
  let seen = {};
  field.subfields = field.subfields.filter((sf, i) => notSeenBefore(sf, i));
  if (field.collapsed) {
    (0, _sortSubfields.sortAdjacentSubfields)(field);
    (0, _punctuation.fieldFixPunctuation)(field);
    delete field.collapsed;
  }
  function notSeenBefore(sf, index) {
    const subfieldAsString = (0, _utils.subfieldToString)(strippedField.subfields[index]); // use normalized form
    if (seen[subfieldAsString]) {
      (0, _utils.nvdebug)(`Remove field-internal duplicate subfield ${(0, _utils.subfieldToString)(sf)}`, debugDev);
      field.collapsed = 1; // trigger punctuation reset
      return false;
    }
    //nvdebug(`identical subfield removal: Add ${subfieldAsString} to seen[]`, debugDev);
    seen[subfieldAsString] = subfieldAsString;
    return true;
  }
  /* eslint-enable */
}
//# sourceMappingURL=removeDuplicateSubfields.js.map