"use strict";

var _chai = _interopRequireDefault(require("chai"));
var _chaiAsPromised = _interopRequireDefault(require("chai-as-promised"));
var _marcRecord = require("@natlibfi/marc-record");
var _fetchMock = _interopRequireDefault(require("fetch-mock"));
var testContext = _interopRequireWildcard(require("../src/resolvable-ext-references-melinda"));
var _resolvableExtReferencesMelinda2 = require("../test-fixtures/resolvable-ext-references-melinda");
function _interopRequireWildcard(e, t) { if ("function" == typeof WeakMap) var r = new WeakMap(), n = new WeakMap(); return (_interopRequireWildcard = function (e, t) { if (!t && e && e.__esModule) return e; var o, i, f = { __proto__: null, default: e }; if (null === e || "object" != typeof e && "function" != typeof e) return f; if (o = t ? n : r) { if (o.has(e)) return o.get(e); o.set(e, f); } for (const t in e) "default" !== t && {}.hasOwnProperty.call(e, t) && ((i = (o = Object.defineProperty) && Object.getOwnPropertyDescriptor(e, t)) && (i.get || i.set) ? o(f, t, i) : f[t] = e[t]); return f; })(e, t); }
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const {
  expect
} = _chai.default;
_chai.default.use(_chaiAsPromised.default);
const endpoint = 'http://melinda.kansalliskirjasto.fi:210/fin01';
const queryParam = '?operation=searchRetrieve&maximumRecords=2&version=1&query=rec.id=';
const prefixPattern = /^\(FOOBAR\)/u;
const fields = {
  773: ['w'],
  833: ['w', 'p']
};
describe('resolvable-ext-references-melinda', () => {
  afterEach(() => {
    testContext.default.__ResetDependency__('fetch');
  });
  it('Creates a validator', async () => {
    const validator = await testContext.default({
      endpoint,
      prefixPattern,
      fields
    });
    expect(validator).to.be.an('object').that.has.any.keys('description', 'validate');
    expect(validator.description).to.be.a('string');
    expect(validator.validate).to.be.a('function');
  });
  it('Throws an error when prefixPattern not provided', async () => {
    const validator = await testContext.default({
      endpoint,
      prefixPattern,
      fields
    });
    // Cannot read property 'fields' of undefined or Cannot read properties of undefined (reading 'fields')'
    await expect(validator.validate()).to.be.rejectedWith(Error, /^Cannot read propert/u);
  });
  describe('#validate', () => {
    it('Finds prefixPattern on record and removes it', async () => {
      const mock = _fetchMock.default.sandbox();
      mock.get(`${endpoint}${queryParam}5000`, _resolvableExtReferencesMelinda2.fixture5000);
      mock.get(`${endpoint}${queryParam}9550`, _resolvableExtReferencesMelinda2.fixture9550);
      testContext.default.__Rewire__('fetch', mock);
      const validator = await testContext.default({
        endpoint,
        prefixPattern,
        fields
      });
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '001',
          value: '123456'
        }, {
          tag: '035',
          subfields: [{
            code: 'a',
            value: '(FI-MELINDA)123456'
          }]
        }, {
          tag: '773',
          subfields: [{
            code: 'w',
            value: '(FOOBAR)5000'
          }]
        }, {
          tag: '833',
          subfields: [{
            code: 'p',
            value: '(FOOBAR)9550'
          }, {
            code: 'c',
            value: '(FI-MELINDA)8850'
          }]
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: true,
        messages: []
      });
    });
    it('Finds no matching prefixPattern on record', async () => {
      const mock = _fetchMock.default.sandbox();
      mock.get(`${endpoint}5000`, _resolvableExtReferencesMelinda2.fixture5000);
      testContext.default.__Rewire__('fetch', mock);
      const validator = await testContext.default({
        endpoint,
        prefixPattern,
        fields
      });
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '001',
          value: '123456'
        }, {
          tag: '035',
          subfields: [{
            code: 'a',
            value: '(FI-MELINDA)123456'
          }]
        }, {
          tag: '773',
          subfields: [{
            code: 'w',
            value: '(FI-MELINDA)123456'
          }]
        }, {
          tag: '833',
          subfields: [{
            code: 'p',
            value: '(FI-MELINDA)2620'
          }, {
            code: 'w',
            value: '(FI-MELINDA)8850'
          }]
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: true,
        messages: []
      });
    });
    it('Finds prefixPattern on record but values not resolvable', async () => {
      const mock = _fetchMock.default.sandbox();
      mock.get(`${endpoint}${queryParam}1000`, _resolvableExtReferencesMelinda2.fixture1000);
      testContext.default.__Rewire__('fetch', mock);
      const validator = await testContext.default({
        endpoint,
        prefixPattern,
        fields
      });
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '001',
          value: '123456'
        }, {
          tag: '035',
          subfields: [{
            code: 'a',
            value: '(FI-MELINDA)123456'
          }]
        }, {
          tag: '773',
          subfields: [{
            code: 'w',
            value: '(FOOBAR)1000'
          }]
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: false,
        messages: ['Field 773$w with value 1000 is not resolvable']
      });
    });
  });
});
//# sourceMappingURL=resolvable-ext-references-melinda.spec.js.map