"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
exports.sortAdjacentRelatorTerms = sortAdjacentRelatorTerms;
exports.tagToRelatorTermSubfieldCode = tagToRelatorTermSubfieldCode;
var _clone = _interopRequireDefault(require("clone"));
var _utils = require("./utils");
var _punctuation = require("./punctuation2");
var _sortFields = require("./sortFields");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// Validator/fixer for sorting $e relator term subfields
//
// Author(s): Nicholas Volk

//import createDebugLogger from 'debug';

//const debug = createDebugLogger('@natlibfi/marc-record-validators-melinda:sortRelatorTerms');
//const debugData = debug.extend('data');

const WORST_WORK = 81;
function _default() {
  return {
    description: 'Sort adjacent $e subfields in field [1678][01]0',
    validate,
    fix
  };
  function fix(record) {
    const res = {
      message: [],
      fix: [],
      valid: true
    };
    const typeOfMaterial = recordToTypeOfMaterial(record);
    record.fields.forEach(field => {
      sortAdjacentRelatorTerms(field, typeOfMaterial);
    });
    return res;
  }
  function validate(record) {
    const res = {
      message: []
    };
    const typeOfMaterial = recordToTypeOfMaterial(record);
    record.fields.forEach(field => {
      const clonedField = (0, _clone.default)(field);
      sortAdjacentRelatorTerms(clonedField, typeOfMaterial);
      const clonedFieldAsString = (0, _utils.fieldToString)(clonedField);
      const fieldAsString = (0, _utils.fieldToString)(field);
      if (fieldAsString !== clonedFieldAsString) {
        // eslint-disable-line functional/no-conditional-statements
        res.message.push(`${fieldAsString} => ${clonedFieldAsString}`); // eslint-disable-line functional/immutable-data
      }
    });
    res.valid = !(res.message.length >= 1); // eslint-disable-line functional/immutable-data
    return res;
  }
}
function recordToTypeOfMaterial(record) {
  if (!record.leader) {
    return undefined;
  }
  if (record.leader.charAt(6) === 'i') {
    // Audio books should follow rules of a book, I guess...
    return 'BK';
  }
  return record.getTypeOfMaterial();
}
function tagToRelatorTermSubfieldCode(tag) {
  if (['100', '110', '600', '610', '700', '710', '720', '751', '752', '800', '810'].includes(tag)) {
    return 'e';
  }
  if (['111', '611', '711', '811'].includes(tag)) {
    return 'j';
  }
  return '?'; // No need to complain. Nothing is found.
}
function isRelatorTermTag(tag) {
  // NV: 111/711, 751 and 752 are very rare
  return ['e', 'j'].includes(tagToRelatorTermSubfieldCode(tag));
}
function swapRelatorTermSubfields(field, typeOfMaterial = undefined) {
  if (!field.subfields) {
    return;
  }
  const subfieldCode = tagToRelatorTermSubfieldCode(field.tag);

  //console.log(`Processing ${fieldToString(field)}`); // eslint-disable-line no-console

  const swapPosition = field.subfields.findIndex((subfield, index) => isSwappable(subfield, index));
  if (swapPosition > 0) {
    swapRelatorTermPair(swapPosition);
    swapRelatorTermSubfields(field, typeOfMaterial); // uh, evil recursion...
    return;
  }

  //console.log(`END ${fieldToString(field)}`); // eslint-disable-line no-console

  function swapRelatorTermPair(index) {
    //console.log(` SWAP`); // eslint-disable-line no-console

    // Swap:
    const tmp = field.subfields[index - 1];
    field.subfields[index - 1] = field.subfields[index]; // eslint-disable-line functional/immutable-data
    field.subfields[index] = tmp; // eslint-disable-line functional/immutable-data
    (0, _punctuation.fieldFixPunctuation)(field);
    return true;
  }
  function isSwappable(sf, index) {
    // NB! we should fix 'e' to 'e' or 'j'....
    if (index === 0 || sf.code !== subfieldCode) {
      return false;
    }
    const currScore = (0, _sortFields.scoreRelatorTerm)(sf.value, typeOfMaterial);
    if (currScore === 0) {
      return false;
    }
    const prevSubfield = field.subfields[index - 1];
    if (prevSubfield.code !== subfieldCode) {
      return false;
    }
    const prevScore = (0, _sortFields.scoreRelatorTerm)(prevSubfield.value, typeOfMaterial);
    // console.log(`PREV: ${prevSubfield.value}/${prevScore}, CURR: ${sf.value}/${currScore}`); // eslint-disable-line no-console
    // If this subfield maps to a Work, then subfields can be swapped, even if we don't have a score for the prev subfield!
    if (prevScore === 0 && currScore < WORST_WORK) {
      return false;
    }
    return currScore > prevScore;
  }
}
function sortAdjacentRelatorTerms(field, typeOfMaterial = undefined) {
  if (!field.subfields || !isRelatorTermTag(field.tag)) {
    return field;
  }
  swapRelatorTermSubfields(field, typeOfMaterial);
  return field;
}
//# sourceMappingURL=sortRelatorTerms.js.map