"use strict";

var _chai = require("chai");
var _marcRecord = require("@natlibfi/marc-record");
var _stripPunctuation = _interopRequireDefault(require("./stripPunctuation"));
var _fixura = require("@natlibfi/fixura");
var _fixugen = _interopRequireDefault(require("@natlibfi/fixugen"));
var _debug = _interopRequireDefault(require("debug"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
(0, _fixugen.default)({
  callback,
  path: [__dirname, '..', 'test-fixtures', 'strip-punctuation'],
  useMetadataFile: true,
  recurse: false,
  fixura: {
    reader: _fixura.READERS.JSON
  },
  mocha: {
    before: () => testValidatorFactory()
  }
});
const debug = (0, _debug.default)('@natlibfi/marc-record-validators-melinda/stripPunctuation:test');
async function testValidatorFactory() {
  const validator = await (0, _stripPunctuation.default)();
  (0, _chai.expect)(validator).to.be.an('object').that.has.any.keys('description', 'validate');
  (0, _chai.expect)(validator.description).to.be.a('string');
  (0, _chai.expect)(validator.validate).to.be.a('function');
}
async function callback({
  getFixture,
  enabled = true,
  fix = false
}) {
  if (enabled === false) {
    debug('TEST SKIPPED!');
    return;
  }
  const validator = await (0, _stripPunctuation.default)();
  const record = new _marcRecord.MarcRecord(getFixture('record.json'));
  const expectedResult = getFixture('expectedResult.json');
  // console.log(expectedResult); // eslint-disable-line

  if (!fix) {
    const result = await validator.validate(record);
    (0, _chai.expect)(result).to.eql(expectedResult);
    return;
  }
  await validator.fix(record);
  (0, _chai.expect)(record).to.eql(expectedResult);
}
//# sourceMappingURL=stripPunctuation.spec.js.map