"use strict";

var _chai = _interopRequireDefault(require("chai"));
var _chaiAsPromised = _interopRequireDefault(require("chai-as-promised"));
var _marcRecord = require("@natlibfi/marc-record");
var _unicodeDecomposition = _interopRequireDefault(require("../src/unicode-decomposition"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const {
  expect
} = _chai.default;
_chai.default.use(_chaiAsPromised.default);
describe('unicode-decomposition', () => {
  it('Creates a validator', async () => {
    const validator = await (0, _unicodeDecomposition.default)();
    expect(validator).to.be.an('object').that.has.any.keys('description', 'validate');
    expect(validator.description).to.be.a('string');
    expect(validator.validate).to.be.a('function');
  });
  describe('#validate', () => {
    it('Finds the record valid', async () => {
      const validator = await (0, _unicodeDecomposition.default)();
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '245',
          subfields: [{
            code: 'a',
            value: 'Föö, Bär'
          }]
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: true,
        messages: []
      });
    });
    it('Finds the record invalid', async () => {
      const validator = await (0, _unicodeDecomposition.default)();
      const record = new _marcRecord.MarcRecord({
        fields: [{
          tag: '001',
          ind1: ' ',
          ind2: '0',
          subfields: [{
            code: 'a',
            value: 'Föö, Bär'
          }]
        }]
      });
      const result = await validator.validate(record);
      expect(result).to.eql({
        valid: false,
        messages: ['The following subfields are not properly decomposed: a']
      });
    });
    describe('#fix', () => {
      it('Should fix the record', async () => {
        const validator = await (0, _unicodeDecomposition.default)();
        const record = new _marcRecord.MarcRecord({
          fields: [{
            tag: '245',
            subfields: [{
              code: 'a',
              value: 'Föö, Bär'
            }, {
              code: 'b',
              value: '== Fubar'
            }]
          }]
        });
        const recordOriginal = record.toObject();
        const fieldModified = {
          tag: '245',
          ind1: ' ',
          ind2: ' ',
          subfields: [{
            code: 'a',
            value: 'Föö, Bär'
          }, {
            code: 'b',
            value: '== Fubar'
          }]
        };
        await validator.fix(record);
        expect(recordOriginal).to.not.eql(record);
        expect(record.fields).to.eql([fieldModified]);
      });
    });
  });
});
//# sourceMappingURL=unicode-decomposition.spec.js.map